import React, { lazy, Suspense } from "react";
import { Switch } from "react-router-dom";
import { connect } from "react-redux";
import { getUserById } from "./actions/auth";

import { LocalContext, setLanguage } from "./utils/localContext";
import { setCssVariables } from "./utils/setSCCvariables";
import { GLOBAL_DEFAULT_THEME } from "./contants";

import { PrivateRouter } from "./privateRouter";
import { decodeJWT } from "./utils/checkJWTToken";

export const ROUTERS = [
	{
		id: 1,
		link: {
			path: "/",
			title: "Main"
		},
		hasAccess: [0, 1],
		exact: true,
		path: "/",
		protected: true,
		component: lazy(() => import("./containers/main"))
	},
	{
		id: 2,
		link: {
			path: "/auth",
			title: "Auth"
		},
		hasAccess: [0, 1],
		exact: true,
		path: "/auth",
		protected: false,
		component: lazy(() => import("./containers/auth"))
	},
	{
		id: 3,
		exact: true,
		link: {
			path: "/redux-todo",
			title: "Redux-todo"
		},
		hasAccess: [0, 1],
		path: "/redux-todo",
		protected: false,
		component: lazy(() => import("./containers/todoRedux"))
	},
	{
		id: 4,
		link: {
			path: "/todo",
			title: "Todo"
		},
		hasAccess: [0, 1],
		exact: true,
		path: "/todo",
		protected: false,
		component: lazy(() => import("./containers/todo"))
	},
	{
		id: 10,
		link: {
			path: "/redux-form",
			title: "Redux Form"
		},
		hasAccess: [0, 1],
		exact: true,
		path: "/redux-form",
		protected: false,
		component: lazy(() => import("./containers/redux-form"))
	},
	{
		id: 5,
		link: {
			path: "/thunk-todo",
			title: "Thunk TODO"
		},
		hasAccess: [0, 1],
		exact: true,
		path: "/thunk-todo",
		protected: false,
		component: lazy(() => import("./containers/thunkTodo"))
	},
	{
		id: 6,
		link: null,
		exact: true,
		hasAccess: [0, 1],
		path: "/item/:id",
		protected: false,
		component: lazy(() => import("./containers/item"))
	},
	{
		id: 7,
		link: {
			path: "/hooks",
			title: "Hooks"
		},
		hasAccess: [1],
		exact: true,
		path: "/hooks",
		protected: true,
		component: lazy(() => import("./containers/hooks"))
	},
	{
		id: 8,
		link: {
			path: "/about-us",
			title: "About us"
		},
		hasAccess: [0, 1],
		exact: true,
		path: "/about-us",
		protected: true,
		component: lazy(() => import("./containers/aboutUs"))
	},
	{
		id: 9,
		hasAccess: [0, 1],
		exact: true,
		protected: false,
		component: () => <div>Page not found</div>
	}
];

class Router extends React.Component {
	constructor(props) {
		super(props);

		const themeFromLocalStorage = localStorage.getItem("theme");

		this.state = {
			lang: navigator.language,
			langJson: setLanguage(navigator.language),
			theme: themeFromLocalStorage || GLOBAL_DEFAULT_THEME
		};

		setCssVariables(themeFromLocalStorage ? themeFromLocalStorage : GLOBAL_DEFAULT_THEME);
	}

	setLanguage = lang => {
		this.setState({ lang, langJson: setLanguage(lang) });
	};

	setTheme = theme => {
		setCssVariables(theme);
		this.setState({ theme });
		localStorage.setItem("theme", theme);
	};

	componentDidMount() {
		const token = localStorage.getItem("token");

		if (token) {
			const { id } = decodeJWT(token);
			this.props.getUserById({ id, token });
		}
	}

	render() {
		return (
			<LocalContext.Provider
				value={{
					lang: this.state.lang,
					json: this.state.langJson,
					setLanguage: this.setLanguage,
					setCssVariables: this.setTheme,
					theme: this.state.theme
				}}>
				<Suspense fallback={<div>Loading...</div>}>
					<Switch>
						{ROUTERS.map(route => (
							<PrivateRouter
								key={route.id}
								exact={route.exact}
								path={route.path}
								protectedRoute={route.protected}
								component={route.component}
								hasAccess={route.hasAccess}
							/>
						))}
					</Switch>
				</Suspense>
			</LocalContext.Provider>
		);
	}
}

export default connect(
	null,
	{ getUserById }
)(Router);
